import { Component, Host, h, Element, Event, } from "@stencil/core";
import api from "../../utils/api";
import { getAttributes } from "../../utils/getAttributes";
import { filterObject } from "../../utils/filterObject";
/** these common HTML attributes will not be considered for fetching data */
const ignoreAttributeList = [
    "class",
    "id",
    "style",
    "slot",
    "tabindex",
    "title",
];
export class DataIngest {
    constructor() {
        /** contains the filtered attributes with their URLs */
        this.attributes = {};
    }
    componentWillLoad() {
        this.refreshAttributes();
        this.observer = new MutationObserver(() => this.refreshAttributes());
        this.observer.observe(this.host, {
            attributes: true,
            childList: false,
            subtree: false,
        });
    }
    disconnectedCallback() {
        this.observer.disconnect();
    }
    /** reads the attributes and their URLs and matches them with the previous ones */
    refreshAttributes() {
        const oldAttributes = this.attributes;
        this.attributes = filterObject(getAttributes(this.host), ignoreAttributeList);
        Object.keys(this.attributes).forEach((key) => {
            if (this.attributes[key] !== oldAttributes[key]) {
                this.loadData(key);
            }
        });
    }
    /** loads the data from the url supplied via attribute to put it onto the child element */
    loadData(attributeName) {
        const url = this.attributes[attributeName];
        if (url === "")
            return;
        this.startRequest.emit(attributeName);
        const child = this.host.firstElementChild;
        const loading = `${attributeName}Loading`;
        if (child === null) {
            return;
        }
        child[loading] = true;
        api(url)
            .then((result) => {
            child[loading] = false;
            child[attributeName] = result;
        })
            .catch(() => {
            child[loading] = false;
        });
    }
    render() {
        return (h(Host, { class: "ux-data-ingest" },
            h("slot", null)));
    }
    static get is() { return "ux-data-ingest"; }
    static get originalStyleUrls() { return {
        "$": ["data-ingest.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["data-ingest.css"]
    }; }
    static get events() { return [{
            "method": "startRequest",
            "name": "uxStartRequest",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Is fired when a request is sent with the name of the property"
            },
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            }
        }]; }
    static get elementRef() { return "host"; }
}
