import { Component, Host, h, Event, Prop } from "@stencil/core";
import api from "../../utils/api";
export class ToggleButton {
    constructor() {
        /** is the toggle on or off */
        this.on = false;
        /** this id is used to identify similar toggleButtons to synchronise them */
        this.syncId = "";
        /** optional url to be called when the item should be toggled */
        this.toggleUrl = "";
        /** optional url to be called when the item should be untoggled (-> off), can be left empty if it is identical to toggleUrl */
        this.untoggleUrl = "";
        /** text to show on toggle button in "on" state (if used with icon it will be the title attribute) */
        this.textOn = "";
        /** text to show on toggle button in "off" state (if used with icon it will be the title attribute) */
        this.textOff = "";
        /** name of icon to show when in state "on" */
        this.iconOn = "";
        /** name of icon to show when in state "off" */
        this.iconOff = "";
        /** the component sets this to true when an error occurs while communicating with the API */
        this.error = false;
    }
    toggle() {
        const { on } = this;
        this.on = !on;
        this.error = false;
        this.waitForToggle(() => {
            this.didToggle.emit(this.on);
            this.syncOtherButtons();
        }, () => {
            this.on = on;
            this.error = true;
            this.toggleFailed.emit(this.on);
        });
    }
    syncOtherButtons() {
        const { syncId, on } = this;
        if (syncId === "")
            return;
        const sameButtons = document.querySelectorAll(`ux-toggle-button[sync-id=${syncId}]`);
        [].forEach.call(sameButtons, (button) => {
            button.on = on;
        });
    }
    waitForToggle(callback, error) {
        const { toggleUrl, untoggleUrl, on } = this;
        if (toggleUrl === "") {
            callback();
            return;
        }
        const url = on && untoggleUrl !== "" ? untoggleUrl : toggleUrl;
        api(url).then(callback).catch(error);
    }
    render() {
        const baseClass = "ux-toggle-button";
        const { on, textOff, textOn, iconOff, iconOn, error } = this;
        const text = on ? textOn : textOff;
        const hasIcons = iconOn !== "" && iconOff !== "";
        return (h(Host, { class: {
                [baseClass]: true,
                "ux-toggle-button--on": on,
                "ux-toggle-button--off": !on,
                "ux-toggle-button--has-error": error,
            }, onClick: (e) => {
                e.preventDefault();
                this.toggle();
            }, title: hasIcons ? text : "" },
            !hasIcons && text,
            hasIcons && h("ux-icon", { name: on ? iconOn : iconOff })));
    }
    static get is() { return "ux-toggle-button"; }
    static get originalStyleUrls() { return {
        "$": ["toggle-button.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["toggle-button.css"]
    }; }
    static get properties() { return {
        "on": {
            "type": "boolean",
            "mutable": true,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "is the toggle on or off"
            },
            "attribute": "on",
            "reflect": true,
            "defaultValue": "false"
        },
        "syncId": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "this id is used to identify similar toggleButtons to synchronise them"
            },
            "attribute": "sync-id",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "toggleUrl": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "optional url to be called when the item should be toggled"
            },
            "attribute": "toggle-url",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "untoggleUrl": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "optional url to be called when the item should be untoggled (-> off), can be left empty if it is identical to toggleUrl"
            },
            "attribute": "untoggle-url",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "textOn": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "text to show on toggle button in \"on\" state (if used with icon it will be the title attribute)"
            },
            "attribute": "text-on",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "textOff": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "text to show on toggle button in \"off\" state (if used with icon it will be the title attribute)"
            },
            "attribute": "text-off",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "iconOn": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "name of icon to show when in state \"on\""
            },
            "attribute": "icon-on",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "iconOff": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "name of icon to show when in state \"off\""
            },
            "attribute": "icon-off",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "error": {
            "type": "boolean",
            "mutable": true,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "the component sets this to true when an error occurs while communicating with the API"
            },
            "attribute": "error",
            "reflect": true,
            "defaultValue": "false"
        }
    }; }
    static get events() { return [{
            "method": "didToggle",
            "name": "uxToggle",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "when button is clicked - returns the new toggle state as boolean"
            },
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            }
        }, {
            "method": "toggleFailed",
            "name": "uxToggleFailed",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "when call to toggle url failed - returns the original toggle state as boolean"
            },
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            }
        }]; }
}
