import { r as registerInstance, d as getAssetPath, h, H as Host } from './index-6beb7a83.js';

const isStr = (val) => typeof val === "string";

const validateContent = (svgContent) => {
    if (svgContent && typeof document !== "undefined") {
        const div = document.createElement("div");
        div.innerHTML = svgContent;
        // setup this way to ensure it works on IE
        for (let i = div.childNodes.length - 1; i >= 0; i--) {
            if (div.childNodes[i].nodeName.toLowerCase() !== "svg") {
                div.removeChild(div.childNodes[i]);
            }
        }
        // must only have 1 root element
        const svgElm = div.firstElementChild;
        if (svgElm && svgElm.nodeName.toLowerCase() === "svg") {
            const svgClass = svgElm.getAttribute("class") || "";
            svgElm.setAttribute("class", (svgClass + " ux-icon__svg").trim());
            svgElm.removeAttribute("id");
            // remove title element to prevent a bug where the svg title was treated as the page title
            const title = svgElm.querySelector("title");
            if (title !== null) {
                title.remove();
            }
            // root element must be an svg
            // lets double check we've got valid elements
            // do not allow scripts
            if (isValid(svgElm)) {
                return div.innerHTML;
            }
        }
    }
    return "";
};
const isValid = (elm) => {
    if (elm.nodeType === 1) {
        if (elm.nodeName.toLowerCase() === "script") {
            return false;
        }
        for (let i = 0; i < elm.attributes.length; i++) {
            const attr = elm.attributes[i];
            if (typeof attr !== "undefined") {
                const val = elm.attributes[i].name;
                if (isStr(val) && val.toLowerCase().indexOf("on") === 0) {
                    return false;
                }
            }
        }
        for (let i = 0; i < elm.childNodes.length; i++) {
            if (!isValid(elm.childNodes[i])) {
                return false;
            }
        }
    }
    return true;
};

const iconContent = new Map();
const requests = new Map();
const getSvgContent = (url) => {
    // see if we already have a request for this url
    let req = requests.get(url);
    if (!req) {
        // we don't already have a request
        req = fetch(url).then((rsp) => {
            if (rsp.ok) {
                return rsp.text().then((svgContent) => {
                    iconContent.set(url, validateContent(svgContent));
                });
            }
            iconContent.set(url, "");
        });
        // cache for the same requests
        requests.set(url, req);
    }
    return req;
};

const availableIcons = [
    "arrow-down",
    "arrow-right",
    "arrow-small-right",
    "arrow-small-up",
    "cart",
    "cart-bending-tools",
    "cart-genuine-parts",
    "cart-punching-tools",
    "case-in-progress",
    "circle-filled",
    "close",
    "checkmark-rounded-thin",
    "cog",
    "crane",
    "documents",
    "fav-filled",
    "fav",
    "hourglass-circle-arrow",
    "list",
    "machine-fav",
    "machine",
    "mail-approved",
    "mail-denied",
    "order-status-package-checkmark",
    "sort",
    "tiles",
    "watchlist",
    "wrench",
    "wrench-checkmark",
    "wrench-plus",
];

const iconCss = ".ux-icon{display:inline-block;vertical-align:middle}.ux-icon__inner{width:1em;height:1em;display:block}.ux-icon__svg{width:100%;height:100%;display:block}";

/**
 * this component is heavily inspired by https://github.com/ionic-team/ionicons
 */
class Icon {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        /** name of icon */
        this.name = "";
        /** contains the full svg */
        this.svgContent = null;
    }
    connectedCallback() {
        this.loadIcon();
    }
    loadIcon() {
        if (this.name === "") {
            return;
        }
        if (availableIcons.indexOf(this.name) === -1) {
            if (loggedWarnings.indexOf(this.name) !== -1)
                return;
            console.warn(`Icon ${this.name} is not available in ux-icon.`);
            loggedWarnings.push(this.name);
            return;
        }
        const url = getAssetPath(`./assets/${this.name}.svg`);
        if (iconContent.has(url)) {
            // sync if it's already loaded
            this.svgContent = iconContent.get(url);
        }
        else {
            // async if it hasn't been loaded
            getSvgContent(url).then(() => (this.svgContent = iconContent.get(url)));
        }
    }
    render() {
        if (this.name === "" || this.svgContent === null) {
            return null;
        }
        return (h(Host, { class: "ux-icon" }, h("div", { class: "ux-icon__inner", innerHTML: this.svgContent })));
    }
    static get assetsDirs() { return ["assets"]; }
    static get watchers() { return {
        "name": ["loadIcon"]
    }; }
}
// this array holds logged warnings about unknown icons to prevent duplicate warnings
const loggedWarnings = [];
Icon.style = iconCss;

export { Icon as ux_icon };
