import { r as registerInstance, h, H as Host, g as getElement } from './index-6beb7a83.js';
import { a as applyPolyfills } from './element-f64f78c4.js';
import { t as throttle } from './throttle-2fb7d237.js';

/** Avoid running the same function twice within the specified timeframe. */
const debounce = (func, wait, immediate = false) => {
    let timeout;
    return function () {
        var context = this, args = arguments;
        clearTimeout(timeout);
        timeout = setTimeout(() => {
            timeout = null;
            if (!immediate)
                func.apply(context, args);
        }, wait);
        if (immediate && !timeout)
            func.apply(context, args);
    };
};

const tableCss = ".ux-table{display:block;position:relative}.ux-table table{border-collapse:collapse;border-spacing:0;margin:0 auto;width:-webkit-max-content;width:-moz-max-content;width:max-content}.ux-table td,.ux-table th{padding:30px;text-align:left;white-space:normal}.ux-table td{font-family:\"Frutiger-Light\", sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;text-rendering:optimizeLegibility;font-weight:400;font-size:16px;text-transform:none}.ux-table th{font-family:\"Frutiger-Bold\", sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;text-rendering:optimizeLegibility;font-weight:400;font-size:13px;color:#fff;text-transform:uppercase;background-color:#899fb2}.ux-table thead tr{background-color:#899fb2}.ux-table tr:nth-child(odd) td{background-color:#eaedf2}.ux-table tr:nth-child(even) td{background-color:#fff}.ux-table__inner{overflow-x:auto;height:auto}.ux-table__inner::before,.ux-table__inner::after{content:\"\";opacity:0;position:absolute;z-index:1;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);width:50px;height:calc(100% + 1px);pointer-events:none;-webkit-transition:opacity 0.5s ease;transition:opacity 0.5s ease}.ux-table__inner::before{background:-webkit-gradient(linear, right top, left top, color-stop(0, rgba(255, 255, 255, 0)), to(rgba(0, 0, 0, 0.2)));background:linear-gradient(to left, rgba(255, 255, 255, 0) 0, rgba(0, 0, 0, 0.2) 100%)}.ux-table__inner::after{right:0;background:-webkit-gradient(linear, left top, right top, color-stop(0, rgba(255, 255, 255, 0.1)), to(#fff));background:linear-gradient(to right, rgba(255, 255, 255, 0.1) 0, #fff 100%)}.ux-table__fixed-link{padding:9px 2px 9px 12px;-webkit-transform:translate(-50%, -50%);transform:translate(-50%, -50%);position:absolute;right:0;top:50%;display:none}.ux-table--has-left-shadow .ux-table__inner::before{opacity:1}.ux-table--has-right-shadow .ux-table__inner::after{opacity:1}.ux-table--full-width table{min-width:100%}.ux-table--is-fixed table{min-width:auto}.ux-table--is-fixed .ux-table__fixed-link{display:inline-block}";

applyPolyfills();
class Table {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        /** zero-based index of the column that should be fixed on smaller viewports - defaults to the first col */
        this.fixedColIndex = 0;
        /** maximum number of pixels that the fixed column is allowed to be wide */
        this.maxFixedColWidth = 250;
        /** the table should always be at least 100% wide */
        this.fullWidth = false;
        /** the table should sort its static data by itself. Do not use this if the data needs to be sorted by some logic (i.e. via an API) */
        this.sortable = false;
        /** The link in the last column will be added into the fixed column on smaller viewports */
        this.hasFixedLink = false;
        /** is true when the table is horizontally scrollable (has a fixed col) */
        this.isFixed = false;
        /** width of the fixed col */
        this.fixedColWidth = 250;
        /** the left shadow is visible */
        this.leftShadow = false;
        /** the right shadow is visible */
        this.rightShadow = false;
        this.contentObserver = new MutationObserver(debounce(() => this.onContentChange(), 100));
        this.onResize = throttle(() => this.calculateTableState(), 200);
        this.onScroll = throttle(() => this.calculateShadows(), 10);
    }
    connectedCallback() {
        this.uniqueId = `ux-table-id-${idCounter++}`;
        this.table = this.element.querySelector("table");
        window.addEventListener("resize", this.onResize);
    }
    componentDidLoad() {
        this.addFixedLink();
        this.calculateFixedColWidth();
        this.calculateTableState();
        this.calculateShadows();
        this.syncRowHeight();
        this.contentObserver.observe(this.innerDiv, {
            childList: true,
            subtree: true,
        });
    }
    disconnectedCallback() {
        window.removeEventListener("resize", this.onResize);
        this.contentObserver.disconnect();
    }
    onContentChange() {
        this.addFixedLink();
        this.syncRowHeight();
        this.calculateFixedColWidth();
        this.calculateTableState();
        this.calculateShadows();
    }
    calculateTableState() {
        const containerWidth = this.element.getBoundingClientRect().width;
        const tableRect = this.table.getBoundingClientRect();
        const tableWidth = tableRect.width + (this.isFixed ? this.fixedColWidth : 0);
        this.isFixed = containerWidth < tableWidth;
    }
    calculateFixedColWidth() {
        const { table, fixedColIndex, maxFixedColWidth } = this;
        const fixedCells = table.querySelectorAll(`td:nth-child(${fixedColIndex + 1})
            ,th:nth-child(${fixedColIndex + 1})`);
        const fixedCellsArray = [].slice.call(fixedCells);
        let width = 0;
        // we only calculate the width for the first 50 items to prevent that bigger tables get slow.
        fixedCellsArray
            .slice(0, 50)
            .forEach((cell) => {
            const cellWidth = cell.offsetWidth;
            width = cellWidth > width ? cellWidth : width;
        });
        this.fixedColWidth =
            width > maxFixedColWidth ? maxFixedColWidth : width;
    }
    calculateShadows() {
        const scrollLeft = this.innerDiv.scrollLeft;
        const containerWidth = this.innerDiv.getBoundingClientRect().width;
        const tableWidth = this.table.getBoundingClientRect().width +
            (this.isFixed ? this.fixedColWidth : 0);
        this.leftShadow = this.isFixed && scrollLeft > 0;
        this.rightShadow = scrollLeft + containerWidth < tableWidth;
    }
    /** the height of multi-line cells might differ from the fixed cell, so we need to sync it to prevent gaps */
    syncRowHeight() {
        const rows = this.table.querySelectorAll("tr");
        [].forEach.call(rows, (row) => {
            const rowHeight = row.getBoundingClientRect().height;
            const fixedCell = row.querySelector(`th:nth-child(${this.fixedColIndex + 1}),td:nth-child(${this.fixedColIndex + 1})`);
            const fixedCellHeight = fixedCell.getBoundingClientRect().height;
            const height = Math.max(rowHeight, fixedCellHeight);
            row.style.height = `${height}px`;
            fixedCell.style.height = `${height}px`;
        });
    }
    sortRows(e) {
        if (!this.sortable)
            return;
        const sorter = e.target;
        const th = sorter.closest("th");
        const titleRow = th.parentElement;
        const colIndex = [].indexOf.call(titleRow.children, th);
        const table = th.closest("table");
        const sortAttr = "data-sort";
        const getCellValue = (tr, idx) => tr.children[idx].hasAttribute(sortAttr)
            ? tr.children[idx].getAttribute(sortAttr)
            : tr.children[idx].textContent;
        const comparer = (idx, asc) => (a, b) => getCellValue(asc ? a : b, idx).localeCompare(getCellValue(asc ? b : a, idx));
        [].slice
            .call(table.querySelectorAll(`tr`))
            .filter((tr) => !tr.isSameNode(titleRow))
            .sort(comparer(colIndex, sorter.sortedDirection === "asc"))
            .forEach((tr) => table.appendChild(tr));
    }
    addFixedLink() {
        if (!this.hasFixedLink)
            return;
        const { table } = this;
        const tableRows = table.querySelectorAll(`tr`);
        const tableRowsArray = [].slice.call(tableRows);
        const fixedLinkClass = "ux-table__fixed-link";
        tableRowsArray.forEach((tableRow) => {
            const lastCellIndex = tableRow.children.length - 1;
            const lastRowCell = tableRow.children[lastCellIndex];
            const link = lastRowCell.querySelector("a");
            if (!link)
                return;
            const firstRowCell = tableRow.children[0];
            const hasFixedLink = firstRowCell.querySelector(`.${fixedLinkClass}`);
            if (!hasFixedLink) {
                const clonedLink = link.cloneNode(true);
                clonedLink.className = fixedLinkClass;
                clonedLink.innerHTML = '<ux-icon name="arrow-right"></ux-icon>';
                firstRowCell.insertAdjacentElement("beforeend", clonedLink);
            }
        });
    }
    render() {
        const baseClass = "ux-table";
        const fixedClass = `${baseClass}--is-fixed`;
        const { uniqueId, isFixed, fixedColIndex, fixedColWidth, maxFixedColWidth, leftShadow, rightShadow, fullWidth, } = this;
        const fixedCol = fixedColIndex + 1; // CSS selectors work with an 1-based index
        return (h(Host, { class: {
                [baseClass]: true,
                [fixedClass]: isFixed,
                [`${baseClass}--has-left-shadow`]: leftShadow,
                [`${baseClass}--has-right-shadow`]: rightShadow,
                [`${baseClass}--full-width`]: fullWidth,
            } }, h("style", null, `
                    #${uniqueId} th:nth-child(${fixedCol}),
                    #${uniqueId} td:nth-child(${fixedCol}) {
                        max-width: ${maxFixedColWidth}px;
                    }
                    .${fixedClass} #${uniqueId} th:nth-child(${fixedCol}),
                    .${fixedClass} #${uniqueId} td:nth-child(${fixedCol}) {
                        position: absolute;
                        left: 0;
                        width: ${fixedColWidth}px;
                    }
                    .${fixedClass} #${uniqueId} table {
                        margin-left: ${fixedColWidth}px;
                    }
                    #${uniqueId}:before {
                        left: ${fixedColWidth}px;
                    }
                `), h("div", { class: `${baseClass}__inner`, ref: (el) => (this.innerDiv = el), onScroll: () => this.onScroll(), id: uniqueId }, h("slot", null))));
    }
    get element() { return getElement(this); }
    static get watchers() { return {
        "isFixed": ["calculateShadows", "syncRowHeight"]
    }; }
}
/** this variable is incremented by each instance */
let idCounter = 0;
Table.style = tableCss;

export { Table as ux_table };
